@@ -1,11 +1,10 @@
-/* Time-stamp: <2007-07-18 10:11:17 poser>
+/* Time-stamp: <2008-03-25 18:24:14 poser>
  *
  * Copyright (C) 2007 William J. Poser (billposer@alum.mit.edu)
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of version 3 of the GNU General Public License as published by
+ * the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +13,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  * or go to the web page:  http://www.gnu.org/licenses/gpl.txt.
  */
 
+/* This file contains SetFormat, the function that determines the ascii format to be read or
+ * written, the associated help function ListFormatArguments, and CountSlots, which is
+ * used to validate user-supplied formats.
+ */
+
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,6 +38,8 @@
 #include "exitcode.h"
 
 
+/* Ancillary function */
+
 int mystrcasecmp(char *a, char *b) {
   char ac;
   char bc;
@@ -47,6 +53,10 @@ int mystrcasecmp(char *a, char *b) {
   return (0);
 }
 
+/* This is the public function. Note that only the first argument is an input parameter.
+ * The other arguments are output parameters.
+*/
+
 void SetFormat(char *fs,
 	       int *FType,
 	       short *UseEntitiesP,
@@ -134,9 +144,11 @@ void SetFormat(char *fs,
       *FType = BSLUD;
       *BMPSplitP = 0;
       return;
+      /*
     case 'W':
       *FType = HDML;
       return;
+      */
     case 'X':
       *FType = STDX;
       return;
@@ -161,6 +173,13 @@ void SetFormat(char *fs,
     case '6':
       *FType = ADA; 
       return;
+    case '7':
+      *FType = APACHE;
+      *UTF8Type = 4;
+      return;
+    case '8':
+      *FType = OOXML;
+      return;
     default:
       fprintf(stderr,"Unknown format %s\n",fs);
       exit(BADOPTIONARG);
@@ -172,7 +191,7 @@ void SetFormat(char *fs,
   if(mystrcasecmp(fs,"RFC2396") == 0) {*FType = JFMT;*UTF8Type = 1;return;}
   if( (mystrcasecmp(fs,"RFC2045") == 0) || 
       (mystrcasecmp(fs,"quoted_printable") == 0)) {*FType = IFMT; *UTF8Type = 2;return;}
-  if(mystrcasecmp(fs,"HDML") == 0) {*FType = HDML;return;}
+  /*  if(mystrcasecmp(fs,"HDML") == 0) {*FType = HDML;return;} */
   if(mystrcasecmp(fs,"HTML_hexadecimal") == 0) {*FType = HTMLX;return;}
   if(mystrcasecmp(fs,"HTML_decimal") == 0) {*FType = HTMLD;return;}
   if(mystrcasecmp(fs,"SGML_hexadecimal") == 0) {*FType = SGMLX;return;}
@@ -181,6 +200,8 @@ void SetFormat(char *fs,
   if(mystrcasecmp(fs,"Python") == 0) {*FType = BSLU;return;}
   if(mystrcasecmp(fs,"RTF") == 0) {*FType = BSLUD;*BMPSplitP = 0;return;}
   if(mystrcasecmp(fs,"Unicode") == 0) {*FType = UPLX;return;}
+  if(mystrcasecmp(fs,"Apache") == 0) {*FType = APACHE;*UTF8Type=4;return;}
+  if(mystrcasecmp(fs,"OOXML") == 0) {*FType = OOXML;return;}
 
 
   /* Match regexps against examples */
@@ -201,9 +222,9 @@ void SetFormat(char *fs,
   if(!regexec(&rx,fs,0,NULL,0)) {*FType = SGMLD;return;}
   regfree(&rx);
 
-  regcomp(&rx,"^&#[[:xdigit:]]{2};$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* HDML */
-  if(!regexec(&rx,fs,0,NULL,0)) {*FType = HDML;return;}
-  regfree(&rx);
+  /*  regcomp(&rx,"^&#[[:xdigit:]]{2};$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	HDML */
+  /*  if(!regexec(&rx,fs,0,NULL,0)) {*FType = HDML;return;} */
+  /*  regfree(&rx); */
 
   regcomp(&rx,"^[\\]u[[:xdigit:]]{4,}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED); 	/* BSLU */
   if(!regexec(&rx,fs,0,NULL,0)) {*FType = BSLU;return;}
@@ -289,16 +310,25 @@ void SetFormat(char *fs,
   if(!regexec(&rx,fs,0,NULL,0)) {*UseEntitiesP = 1;return;}
   regfree(&rx);
 
-  regcomp(&rx,"^(=[[:xdigit:]]{2}){1,3}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* Non-numeric Character entity */
-  if(!regexec(&rx,fs,0,NULL,0)) {*FType = IFMT;return;}
+  regcomp(&rx,"^(=[[:xdigit:]]{2}){1,3}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* I format */
+  if(!regexec(&rx,fs,0,NULL,0)) {*FType = IFMT;*UTF8Type=2;return;}
   regfree(&rx);
 
-  regcomp(&rx,"^(%[[:xdigit:]]{2}){1,3}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* Non-numeric Character entity */
-  if(!regexec(&rx,fs,0,NULL,0)) {*FType = JFMT;return;}
+  regcomp(&rx,"^(%[[:xdigit:]]{2}){1,3}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* J format */
+  if(!regexec(&rx,fs,0,NULL,0)) {*FType = JFMT;*UTF8Type=1;return;}
+  regfree(&rx);
+
+  regcomp(&rx,"^([\\][0-7]{3}){1,3}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* K format */
+  if(!regexec(&rx,fs,0,NULL,0)) {*FType = KFMT;*UTF8Type=3;return;}
+
+  regcomp(&rx,"^([\\][xX][[:xdigit:]]{2}){1,3}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* Apache */
+  if(!regexec(&rx,fs,0,NULL,0)) {*FType = APACHE;*UTF8Type=4;return;}
+  regfree(&rx);
+
+  regcomp(&rx,"^_[xX][[:xdigit:]]{1,6}_$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* OOXML */
+  if(!regexec(&rx,fs,0,NULL,0)) {*FType = OOXML;return;}
   regfree(&rx);
 
-  regcomp(&rx,"^([\\][0-7]{3}){1,3}$",REG_NEWLINE|REG_NOSUB|REG_EXTENDED);	/* Non-numeric Character entity */
-  if(!regexec(&rx,fs,0,NULL,0)) {*FType = KFMT;return;}
 
   regfree(&rx);
   *FType = FMT_UNKNOWN;
@@ -379,8 +409,8 @@ void OldListFormatArguments (short ToAsciiP) {
 	  _(" U \\u-escaped hex (\\u00E9)\n"));
   fprintf(stderr,
 	  _(" V \\u-escaped decimal (\\u0233)\n"));
-  fprintf(stderr,
-	  _(" W HDML two hexadecimal digits (&#E9;)\n"));
+  /*  fprintf(stderr,
+      _(" W HDML two hexadecimal digits (&#E9;)\n")); */
   fprintf(stderr,
 	  _(" X standard form hexadecimal numbers (0x00E9)\n"));
   if(!ToAsciiP) {
@@ -417,13 +447,14 @@ void ListFormatArguments (short ToAsciiP) {
   fprintf(stderr, _(" HTML numeric character references - hexadecimal (&#x00E9;)\tH\n"));
   fprintf(stderr, _(" SGML numeric character references -decimal (\\#0233;)\tN\n"));
   fprintf(stderr, _(" SGML numeric character references - hexadecimal (\\#x00E9;)\tM\n"));
-  fprintf(stderr, _(" HDML two hexadecimal digits (&#E9;)\tW\n"));
+  /*  fprintf(stderr, _(" HDML two hexadecimal digits (&#E9;)\tW\n")); */
   fprintf(stderr, _(" octal escapes for 3 low bytes in big-endian order (\\000\\000\\351)\tO\n"));
   fprintf(stderr, _(" hexadecimal escapes for 3 low bytes in big-endian order\n"));
   fprintf(stderr,"\t(\\x00\\x00\\xE9)\tS\n");
   fprintf(stderr, _(" decimal escapes for 3 low bytes in big-endian order (\\d000\\d000\\d233)\tT\n"));
   fprintf(stderr, _(" hexadecimal UTF-8 with each byte's hex preceded by an =-sign (=C3=A9).\n\t\tRFC 2045 Quoted Printable.\tI\n"));
   fprintf(stderr, _(" hexadecimal UTF-8 with each byte's hex preceded by a %%-sign  (%%C3%%A9)\n\t\tRFC 2396  URI escape format.\tJ\n"));
+  fprintf(stderr, _(" hexadecimal UTF-8 with each byte's hex preceded by a backslash-x  (\\xC3\\xA9)\n\t\tApache log format.\t7\n"));
   fprintf(stderr, _(" octal UTF-8 with backslash escapes (\303\251)\tK\n"));
   fprintf(stderr, _(" character entities where possible (&eacute;)\tQ\n"));
   if(!ToAsciiP) {
