@@ -24,10 +24,12 @@
 #define BYTEO  22
 #define BYTED  23
 #define BYTEH  24
-#define CHENT  25	/* Used only by ascii2uni */
-#define IFMT   26
-#define JFMT   27
-#define KFMT   28
+#define OOXML  25
+#define CHENT  26	/* Used only by ascii2uni */
+#define IFMT   27
+#define JFMT   28
+#define KFMT   29
+#define APACHE 30
 
 
 
